//********************************************************************************
// Módulo de funções auxiliares e depuração
// 
// Desenvolvido por Márcio José Soares - 07/02/2008 
//
// Microcontrolador: PIC
//
// Adaptado para PICC CCS em 09/03/2017
// Adaptado para XC8 em 18/05/2023
//********************************************************************************

//**********************************************************************
// Arquivos importantes inclúidos no módulo
//**********************************************************************
#include "my_defines.h"

//**********************************************************************
// Definições do módulo 
#ifndef byte
#define byte unsigned char
#endif

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

#define CRC_POLY_CCITT          0x1021  //polinominal do CCITT
#define	CRC_START_CCITT_FFFF	0xFFFF  //

//**********************************************************************
// Variáveis do módulo
#ifdef __INC_CRC__
static Uint16    crc_tabccitt[256];         //tabelapara calculo
static byte      crc_tabccitt_init = 0;     //tabela ainda n�o gerada
#endif

//********************************************************************************
// Funções do módulo
//********************************************************************************

//********************************************************************************
// Função para converter binário para HEX (saída em formato ASCII)
//
// Obs.: Criar de modo global as variáreis high_nibble, low_nibble,
//       high_char e low_char antes de usar esta função!
//
// Entradas - valor a converter, dois ponteiros para retorno dos nibbles
// Saídas   - nenhuma, valores passados por referência
//********************************************************************************
void bin_hex(byte binchar, byte *high_char, byte *low_char){

	byte high_nibble, low_nibble;

	high_nibble = (binchar & 0xF0) / 16;
	if(high_nibble > 0x09)
		*high_char = high_nibble + 0x37;
	else
		*high_char = high_nibble + 0x30;

	low_nibble = (binchar & 0x0F);

	if(low_nibble > 0x09)
		*low_char = low_nibble + 0x37;
	else
		*low_char = low_nibble + 0x30;

}

#ifdef __INC_CRC__
//**********************************************************************
// Função init_crcccitt_tab - inicia tabela de calculo CRCCCITT
//
// Entradas - nenhuma
// Saídas   - nenhuma
//**********************************************************************
void init_crcccitt_tab( void ) {

	Uint16 i;
	Uint16 j;
	Uint16 crc;
	Uint16 c;

	for (i=0; i<256; i++) {

		crc = 0;
		c   = i << 8;

		for (j=0; j<8; j++) {

			if ( (crc ^ c) & 0x8000 )
                crc = ( crc << 1 ) ^ CRC_POLY_CCITT;
			else                      
                crc =   crc << 1;

			c = c << 1;
		}

		crc_tabccitt[i] = crc;
	}

	crc_tabccitt_init = 1;

} 

//**********************************************************************
// Função crc_ccitt_generic - calcula CRCCCITT 
//
// Entradas - buffer com dados, quantida de bytes a calcular, valor para startup
// Saídas   - inteiro de 16bits com valor do CRC
//**********************************************************************
Uint16 crc_ccitt_generic(byte *input_str, Uint16 num_bytes, Uint16 start_value)
{

	Uint16 crc;
	Uint16 tmp;
	Uint16 short_c;
	Uint16 a;
    //byte *ptr;

	if (!crc_tabccitt_init)
        init_crcccitt_tab();

	crc = start_value; 
	
    if (input_str[0] != '\0')
    {
        for (a=0; a<num_bytes; a++) 
        {
            short_c = 0x00ff & (unsigned short) input_str[a];
            tmp     = (crc >> 8) ^ short_c;
            crc     = (crc << 8) ^ crc_tabccitt[tmp];
        }
    }
    else                                //do contr�rio, retorna zero!
    {
        crc = 0;
    }
    
	return crc;

}
#endif
//***********************************************************************
// Função para converter bin�rio->bcd 
// recebe 	- byte a converter 
// retorna 	- byte convertido
//***********************************************************************
byte bin_bcd(byte rec){

   byte lsb, msb, ret;

	if(rec > 9){
  		msb = (rec / 10) & 0x0F;		//pega inteiro da divis�o
		msb <<= 4;							//shift left x4
		lsb = (rec % 10) & 0x0F;		//resto da divis�o
	}
	else{
		msb = 0;				//zero
		lsb = rec;
	}
	
	ret = msb | lsb;	//junta as partes

   return ret;			//retorna byte alterado
}

//******************************************************************************
// Função BCD2BIN - converte BCD para bin�rio
//
// Entradas - dado a converter
// Saídas   - dado convertido
//******************************************************************************
byte BCD2Bin(byte data){
	return ((((data & 0xF0 )>>4) * 10) + (data&0xF));	  
} 

//******************************************************************************
// Função D2BCD - converte dado para BCD
//
// Entradas - dado a converter
// Saídas   - dado convertido
//******************************************************************************
byte D2BCD(byte data){
    if(data > 9)
        return (byte)(((data/10)<< 4) | (data%10));
    else
        return (0 | data);
}		


 
