//******************************************************************************
// pic_SPI.c - Controle SPI manual para PIC
//
// Desenvolvido por: Eng. Márcio José Soares
// versão: 1.0 - 15/10/2022
//
//******************************************************************************

//******************************************************************************
// Arquivos incluídos no módulo
//******************************************************************************
#include "pic_SPI.h"

//******************************************************************************
// Funções do módulo

//******************************************************************************
// SPI_init - inicia pinos de I/O para SPI
//
// Entradas - nenhuma
// Saídas   - nenhuma
//******************************************************************************
void SPI_init(void){
    
    TRISB0 = 0;       //RB0 - saída
    TRISB3 = 0;       //RB3 - Saida
    TRISB7 = 1;       //RB7 - entrada
    TRISA4 = 0;       //RA4 - saída
    
    CS = 1;
    SCK = 0; 
    SDO = 0;
    
}

//*****************************************************************************
// SPI_read - recebe um byte via SPI1 manualmente
//
// Entradas - nenhuma
// Saídas   - byte recolhido de MISO
//*****************************************************************************
uint8_t SPI_read(void){
    
    byte byte_in = 0x00;
    byte bit = 0x80;
       
    for (bit = 0x80; bit; bit >>= 1){
        SCK = 0;
        __delay_us(TVELSPI);              //período T/2
        
        //shift-in do bit na linha MISO
        if(SDI)
            byte_in |= bit;
         
        //coloca o clock em low
        SCK = 1;
        __delay_us(TVELSPI);              //período T/2
        
    }
    
    return (byte_in);
}

//******************************************************************************
// SPI_write - envia 1 byte via SPI
//
// Entradas - byte a enviar
// Saídas   - nenhuma
//******************************************************************************
void SPI_write(uint8_t byte_out){
    
    uint8_t bit=0;
    
    for(bit=0x80; bit; bit >>=1){     
        SCK = 0;            // abaixa clock
        if(byte_out & bit)
            SDO = 1;
        else
            SDO = 0;
        
        SCK = 1;        
    }
    
    SCK = 0;
}

//******************************************************************************
// SPI_writeWord - envia 2 bytes via SPI
//
// Entradas - byte a enviar
// Saídas   - nenhuma
//******************************************************************************
void SPI_writeWord(uint8_t address, uint8_t data){
    
    SPI_write(address);
    SPI_write(data);
}