//*****************************************************************************
// Arquivo pic_timer1.c - módulo de configuração e uso do timer1
// Desenvolvido por Márcio José Soares
//*****************************************************************************

//*****************************************************************************
// O Copyright deste programa está reservado para Márcio José Soares e seu 
// conteúdo está protegido pela Lei de Direitos Autorais LEI Nº 9.610, de 
// 19 de Fevereiro de 1998. É estritamente proibida a reprodução total ou 
// parcial dos conteúdos aqui apresentados sem prévia autorização, por escrito, 
// do autor.
//*****************************************************************************

//*****************************************************************************
// Arquivos incluídos
//*****************************************************************************
#include "my_defines.h"

//*****************************************************************************
// Variáveis externas
//*****************************************************************************
extern volatile uint8_t autoriza;
extern volatile uint8_t counter;
extern volatile uint8_t prima;
extern volatile uint8_t mytimeout;
extern volatile uint8_t timeout_aut;
extern volatile uint8_t toTerm;

//*****************************************************************************
// Funções externas do módulo
//*****************************************************************************
extern uint8_t getBTNs(void);
extern void msgTerm(uint8_t toTerm);

//*****************************************************************************
// Funções do módulo
//*****************************************************************************

//*****************************************************************************
// Função init_timer1 -inicia timer 1
//
// Entradas - nenhuma
// Saidas   - nenhuma
//*****************************************************************************
void init_timer1(void){

   //**************************************************************************
   //Setup do timer com clock de 4MHz 
   //4MHz/4 = 1MHz -> 1/1MHz = 1us -> tempo = 1us x prescaler x contador
   //tempo = 1us x 8 x 6250 = 50ms

   T1CON = 0x30;              	// TMR1CS = 0 - Timer1 é temporizador, prescale = 8
   TMR1H = 0xE7;                // reestabelece valores iniciais do contador
   TMR1L = 0x96;                // assim a temporização fica = 8 us x 6250 = 50 ms
   TMR1IF = 0x00;               // zera  flag

   PIE1 |= 0x01;              	// TMR1IE = 1 habilita int timer 1                  
   T1CON |= 0x01;             	// TMR1ON = 1 - ligado e desligado via int
}


//*****************************************************************************
// Função trata_int_timer1 - realiza tratamento da int do timer1
//
// Entradas - nenhuma
// Saídas   - nenhuma
//*****************************************************************************
void trata_int_timer1(void){

    if(timeout_aut){
       mytimeout++;
        if (mytimeout >= 40){       //50 ms x 40 = 2 segundos
            mytimeout = 0;          // zera contador
            timeout_aut = 0;
            autoriza = 0;
            prima = 0;
            counter = 0;
        }
	} 
    
    toTerm = getBTNs();				// verifica se tem chave 
    if(toTerm){                     // se tem envia msg
        msgTerm(toTerm);
    }
    
    TMR1H = 0xE7;                        // reestabelece valores iniciais do contador
    TMR1L = 0x96                                                                            ;                        // assim a temporização fica = 8 us x 6250 = 50 ms
    TMR1IF = 0;                          // zera  flag
}
